"""Cylindrical color space."""


class Cylindrical:
    """Cylindrical space."""

    def hue_name(self):
        """Hue channel name."""

        return "hue"

    def hue_index(self):
        """Get hue index."""

        return 0
